// -*- mode: typescript; indent-tabs-mode: nil; js-basic-offset: 4 -*-
//
// This file is part of Genie
//
// Copyright 2020 The Board of Trustees of the Leland Stanford Junior University
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//    http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
// Author: Giovanni Campagna <gcampagn@cs.stanford.edu>

import { Ast, } from 'thingtalk';
import * as C from '../ast_manip';
import ThingpediaLoader from '../load-thingpedia';
import type * as $Genie from "genie-toolkit";
export default async function($runtime : typeof $Genie.SentenceGeneratorRuntime, $ttUtils : typeof $Genie.ThingTalkUtils, $options : $Genie.SentenceGeneratorTypes.GrammarOptions, $locale : $Genie.I18n.LanguagePack, $grammar : $Genie.SentenceGenerator<any, any>, $loader : ThingpediaLoader) : Promise<void> {

$loader = new ThingpediaLoader($runtime, $ttUtils, $grammar, $locale, $options);
   await (await $runtime.import("../common.genie", __dirname))($runtime, $ttUtils, $options, $locale, $grammar, $loader);

await $loader.init();

function special(specialType : string) {
    return new Ast.ControlCommand(null, new Ast.SpecialControlIntent(null, specialType));
}
   await (await $runtime.import("./constants.genie", __dirname))($runtime, $ttUtils, $options, $locale, $grammar, $loader);
   $grammar.declareSymbol("bookkeeping");
   $grammar.addRule("bookkeeping", ["debug"], (($0 : undefined) : (Ast.ControlCommand)|null => special('debug')), undefined, {  });
   $grammar.addRule("bookkeeping", ["cancel"], (($0 : undefined) : (Ast.ControlCommand)|null => special('nevermind')), undefined, {  });
   $grammar.addRule("bookkeeping", ["never mind"], (($0 : undefined) : (Ast.ControlCommand)|null => special('nevermind')), undefined, {  });
   $grammar.addRule("bookkeeping", ["stop"], (($0 : undefined) : (Ast.ControlCommand)|null => special('stop')), undefined, {  });
   $grammar.addRule("bookkeeping", ["shut up"], (($0 : undefined) : (Ast.ControlCommand)|null => special('stop')), undefined, {  });
   $grammar.addRule("bookkeeping", ["shush"], (($0 : undefined) : (Ast.ControlCommand)|null => special('stop')), undefined, {  });
   $grammar.addRule("bookkeeping", ["quiet"], (($0 : undefined) : (Ast.ControlCommand)|null => special('stop')), undefined, {  });
   $grammar.addRule("bookkeeping", ["close"], (($0 : undefined) : (Ast.ControlCommand)|null => special('stop')), undefined, {  });
   $grammar.addRule("bookkeeping", [new $runtime.NonTerminal("constant_Any", undefined)], ((c : Ast.Value) : (Ast.ControlCommand)|null => new Ast.ControlCommand(null, new Ast.AnswerControlIntent(null, c))), undefined, {  });
   $grammar.declareSymbol("$root");
   $grammar.addRule("$root", [new $runtime.NonTerminal("bookkeeping", undefined)], (($0 : Ast.ControlCommand) : (Ast.Input)|null => { return $0; }), undefined, { identity: (true) });
   $grammar.addRule("$root", ["get", new $runtime.NonTerminal("thingpedia_complete_query", undefined)], (($0 : undefined, query : Ast.Expression) : (Ast.Input)|null => C.makeProgram($loader, query)), undefined, {  });
   $grammar.addRule("$root", [new $runtime.NonTerminal("thingpedia_complete_get_command", undefined)], ((query : Ast.Expression) : (Ast.Input)|null => C.makeProgram($loader, query)), undefined, {  });
   $grammar.addRule("$root", [new $runtime.NonTerminal("thingpedia_complete_action", undefined)], ((action : Ast.Expression) : (Ast.Input)|null => C.makeProgram($loader, action)), undefined, {  });
   $grammar.addRule("$root", ["notify me", new $runtime.NonTerminal("thingpedia_complete_stream", undefined)], (($0 : undefined, stream : Ast.Expression) : (Ast.Input)|null => C.makeProgram($loader, stream)), undefined, {  });
};
