// -*- mode: js; indent-tabs-mode: nil; js-basic-offset: 4 -*-
//
// This file is part of Genie
//
// Copyright 2019-2020 The Board of Trustees of the Leland Stanford Junior University
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//    http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
// Author: Silei Xu <silei@cs.stanford.edu>

import { Ast, } from 'thingtalk';
import * as C from '../ast_manip';
import ThingpediaLoader from '../load-thingpedia';
import type * as $Genie from "genie-toolkit";
export default async function($runtime : typeof $Genie.SentenceGeneratorRuntime, $ttUtils : typeof $Genie.ThingTalkUtils, $options : $Genie.SentenceGeneratorTypes.GrammarOptions, $locale : $Genie.I18n.LanguagePack, $grammar : $Genie.SentenceGenerator<any, any>, $loader : ThingpediaLoader) : Promise<void> {
   $grammar.declareSymbol("schema_with_filtered_table_join");
   $grammar.addRule("schema_with_filtered_table_join", [new $runtime.NonTerminal("with_filtered_table", undefined), new $runtime.Choice(["with", "having", "that has", "that have", "which has", "which have"]), new $runtime.NonTerminal("with_filtered_table", undefined)], ((table : Ast.Expression, $0 : undefined, get_predicate_table : Ast.Expression) : (Ast.Expression)|null => C.addGetPredicateJoin(table, get_predicate_table, null)), C.expressionKeyFn, {  });
   $grammar.addRule("schema_with_filtered_table_join", [new $runtime.NonTerminal("with_filtered_table", undefined), "with", new $runtime.NonTerminal("out_param_Any", undefined), new $runtime.NonTerminal("with_filtered_table", undefined)], ((table : Ast.Expression, $0 : undefined, p : C.ParamSlot, get_predicate_table : Ast.Expression) : (Ast.Expression)|null => C.addGetPredicateJoin(table, get_predicate_table, p.name)), C.expressionKeyFn, {  });
   $grammar.addRule("schema_with_filtered_table_join", [new $runtime.NonTerminal("with_filtered_table", undefined), new $runtime.NonTerminal("out_param_pvp", undefined), new $runtime.NonTerminal("with_filtered_table", undefined)], ((table : Ast.Expression, p : C.ParamSlot, get_predicate_table : Ast.Expression) : (Ast.Expression)|null => C.addGetPredicateJoin(table, get_predicate_table, p.name)), C.expressionKeyFn, {  });
   $grammar.addRule("schema_with_filtered_table_join", [new $runtime.NonTerminal("with_filtered_table", undefined), new $runtime.Choice(["who", "which", "that"]), new $runtime.NonTerminal("out_param_avp", undefined), new $runtime.NonTerminal("with_filtered_table", undefined)], ((table : Ast.Expression, $0 : undefined, p : C.ParamSlot, get_predicate_table : Ast.Expression) : (Ast.Expression)|null => C.addGetPredicateJoin(table, get_predicate_table, p.name)), C.expressionKeyFn, {  });
   $grammar.addRule("schema_with_filtered_table_join", [new $runtime.NonTerminal("with_filtered_table", undefined), new $runtime.Choice(["of", "for", "in"]), new $runtime.NonTerminal("with_filtered_table", undefined)], ((rhs : Ast.Expression, $0 : undefined, lhs : Ast.Expression) : (Ast.Expression)|null => C.addArrayJoin(lhs, rhs)), C.expressionKeyFn, {  });
   if (!$options.flags.turking) {
       $grammar.addRule("schema_with_filtered_table_join", [new $runtime.NonTerminal("with_filtered_table", undefined), "'s", new $runtime.NonTerminal("with_filtered_table", undefined)], ((lhs : Ast.Expression, $0 : undefined, rhs : Ast.Expression) : (Ast.Expression)|null => C.addArrayJoin(lhs, rhs)), C.expressionKeyFn, {  });
   }
   $grammar.declareSymbol("schema_have_filtered_table_join");
   $grammar.addRule("schema_have_filtered_table_join", [new $runtime.NonTerminal("verb_filtered_table", undefined), new $runtime.Choice(["and", "and have", "and also", "and in addition", "but also"]), new $runtime.NonTerminal("with_filtered_table", undefined)], ((table : Ast.Expression, $0 : undefined, get_predicate_table : Ast.Expression) : (Ast.Expression)|null => C.addGetPredicateJoin(table, get_predicate_table, null)), C.expressionKeyFn, {  });
};
